@@ -1,11 +1,11 @@
-# $Id: TCP.pm,v 1.55 2005/08/19 16:12:45 rcaputo Exp $
+# $Id: TCP.pm 1976 2006-06-06 03:07:35Z immute $
 
 package POE::Component::Server::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.55 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my($r)=(q$Revision: 1976 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_UNIX PF_UNIX);
@@ -18,6 +18,8 @@ use POE::Filter::Line;
 use POE::Wheel::ReadWrite;
 use POE::Wheel::SocketFactory;
 
+sub DEBUG () { 0 }
+
 # Create the server.  This is just a handy way to encapsulate
 # POE::Session->create().  Because the states are so small, it uses
 # real inline coderefs.
@@ -43,17 +45,33 @@ sub new {
   my $hname   = delete $param{Hostname};
   my $port    = delete $param{Port};
   my $domain  = delete $param{Domain};
+  my $concurrency = delete $param{Concurrency};
 
   foreach (
     qw(
       Acceptor Error ClientInput ClientConnected ClientDisconnected
       ClientError ClientFlushed
+      ClientLow ClientHigh
     )
   ) {
     croak "$_ must be a coderef"
       if defined($param{$_}) and ref($param{$_}) ne 'CODE';
   }
 
+  my $high_mark_level = delete $param{HighMark};
+  my $low_mark_level  = delete $param{LowMark};
+  my $high_event      = delete $param{ClientHigh};
+  my $low_event       = delete $param{ClientLow};
+
+  # this is ugly, but now its elegant :)  grep++
+  my $foo = grep { defined $_ } ($high_mark_level, $low_mark_level, $high_event, $low_event);
+  if ($foo > 0) {
+    croak "If you use the Mark settings, you must define all four" unless $foo == 4;
+  }
+
+  $high_event = sub { } unless defined $high_event;
+  $low_event  = sub { } unless defined $low_event;
+
   my $accept_callback = delete $param{Acceptor};
   my $error_callback  = delete $param{Error};
 
@@ -70,7 +88,6 @@ sub new {
     }
   }
 
-  my (@client_filter_args, @client_infilter_args, @client_outfilter_args);
   my $client_connected    = delete $param{ClientConnected};
   my $client_disconnected = delete $param{ClientDisconnected};
   my $client_error        = delete $param{ClientError};
@@ -85,6 +102,9 @@ sub new {
 
   # Defaults.
 
+  $concurrency = -1 unless defined $concurrency;
+  my $accept_session;
+
   if (!defined $address && defined $hname) {
     $address = inet_aton($hname);
   }
@@ -103,56 +123,6 @@ sub new {
   }
 
   if (defined $client_input) {
-    if (defined $client_infilter and defined $client_outfilter) {
-
-      @client_infilter_args  = ();
-      @client_outfilter_args = ();
-
-      if (ref($client_infilter) eq 'ARRAY') {
-         @client_infilter_args = @$client_infilter;
-         $client_infilter      = shift @client_infilter_args;
-      }
-
-      if (ref($client_outfilter) eq 'ARRAY') {
-         @client_outfilter_args = @$client_outfilter;
-         $client_outfilter      = shift @client_outfilter_args;
-      }
-
-      my $eval = eval {
-        (my $inmod  = $client_infilter)  =~ s!::!/!g;
-        (my $outmod = $client_outfilter) =~ s!::!/!g;
-        require "$inmod.pm";
-        require "$outmod.pm";
-        1;
-      };
-
-      if (!$eval and $@) {
-        carp(
-          "Failed to load [$client_infilter] and [$client_outfilter]\n" .
-          "Reason $@\nUsing defualts "
-        );
-
-        undef($client_infilter);
-        undef($client_outfilter);
-        $client_filter      = "POE::Filter::Line";
-        @client_filter_args = ();
-      }
-
-    }
-    else {
-       undef($client_infilter);  # just to be safe in case one was defined
-       undef($client_outfilter); # and the other wasn't
-
-       unless (defined $client_filter) {
-         $client_filter      = "POE::Filter::Line";
-         @client_filter_args = ();
-       }
-       elsif (ref($client_filter) eq 'ARRAY') {
-         @client_filter_args = @$client_filter;
-         $client_filter      = shift @client_filter_args;
-       }
-    }
-
     $client_error  = \&_default_client_error unless defined $client_error;
     $client_connected    = sub {} unless defined $client_connected;
     $client_disconnected = sub {} unless defined $client_disconnected;
@@ -192,6 +162,8 @@ sub new {
     unless (defined $accept_callback) {
       $accept_callback = sub {
         my ($socket, $remote_addr, $remote_port) = @_[ARG0, ARG1, ARG2];
+
+
         $session_type->create(
           @$session_params,
           inline_states => {
@@ -224,38 +196,27 @@ sub new {
 
               $heap->{remote_port} = $remote_port;
 
-              # Alter the filters depending whether they are
-              # discrete input/output ones, or whether they are the
-              # same.  It's useful to note that the input/output
-              # filters may be changed at runtime.
-              my @filters;
-              if ($client_infilter) {
-                @filters = (
-                  InputFilter  => $client_infilter->new(
-                    @client_infilter_args
-                  ),
-                  OutputFilter => $client_outfilter->new(
-                    @client_outfilter_args
-                  ),
-                );
-              }
-              else {
-                @filters = (
-                  Filter => $client_filter->new(@client_filter_args),
-                );
-              }
-
               $heap->{client} = POE::Wheel::ReadWrite->new(
                 Handle       => splice(@_, ARG0, 1),
                 Driver       => POE::Driver::SysRW->new(),
-                @filters,
+                _get_filters($client_filter, $client_infilter, $client_outfilter), 
                 InputEvent   => 'tcp_server_got_input',
                 ErrorEvent   => 'tcp_server_got_error',
                 FlushedEvent => 'tcp_server_got_flush',
+                do {
+                  $foo ? return (
+                    HighMark => $high_mark_level,
+                    HighEvent => 'tcp_server_got_high',
+                    LowMark => $low_mark_level,
+                    LowEvent => 'tcp_server_got_low',
+                  ) : ();
+                },
               );
 
               $client_connected->(@_);
             },
+            tcp_server_got_high => $high_event,
+            tcp_server_got_low => $low_event,
 
             # To quiet ASSERT_STATES.
             _child  => sub { },
@@ -266,6 +227,9 @@ sub new {
               undef;
             },
             tcp_server_got_error => sub {
+              DEBUG and warn(
+                "$$: $alias child Error ARG0=$_[ARG0] ARG1=$_[ARG1]"
+              );
               unless ($_[ARG0] eq 'accept' and $_[ARG1] == ECONNABORTED) {
                 $client_error->(@_);
                 if ($_[HEAP]->{shutdown_on_error}) {
@@ -276,13 +240,16 @@ sub new {
             },
             tcp_server_got_flush => sub {
               my $heap = $_[HEAP];
+              DEBUG and warn "$$: $alias child Flush";
               $client_flushed->(@_);
               if ($heap->{shutdown}) {
+                DEBUG and warn "$$: $alias child Flush, callback";
                 $client_disconnected->(@_);
                 delete $heap->{client};
               }
             },
             shutdown => sub {
+              DEBUG and warn "$$: $alias child Shutdown";
               my $heap = $_[HEAP];
               $heap->{shutdown} = 1;
               if (defined $heap->{client}) {
@@ -290,12 +257,31 @@ sub new {
                   $heap->{got_an_error} or
                   not $heap->{client}->get_driver_out_octets()
                 ) {
+                  DEBUG and warn "$$: $alias child Shutdown, callback";
                   $client_disconnected->(@_);
                   delete $heap->{client};
                 }
               }
             },
-            _stop => sub {},
+            _stop => sub {
+              ## concurrency on close
+              DEBUG and warn(
+                "$$: $alias _stop accept_session = $accept_session"
+              );
+              if( defined $accept_session ) {
+                $_[KERNEL]->call( $accept_session, 'disconnected' );
+              }
+              else {
+                # This means that the Server::TCP was shutdown before
+                # this connection closed.  So it doesn't really matter that
+                # we can't decrement the connection counter.
+                DEBUG and warn(
+                  "$$: $_[HEAP]->{alias} Disconnected from a connection ",
+                  "without POE::Component::Server::TCP parent"
+                );
+              }
+              return;
+            },
 
             # User supplied states.
             %$inline_states
@@ -316,10 +302,30 @@ sub new {
     carp "$mi doesn't recognize \"$_\" as a parameter";
   }
 
+  ## verify concurrency on accept
+  my $orig_accept_callback = $accept_callback;
+  $accept_callback = sub {
+    $_[HEAP]->{connections}++;
+    DEBUG and warn(
+      "$$: $_[HEAP]->{alias} Connection opened ",
+      "($_[HEAP]->{connections} open)"
+    );
+    if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
+      if( $_[HEAP]->{connections} >= $_[HEAP]->{concurrency} ) {
+        DEBUG and warn(
+          "$$: $_[HEAP]->{alias} Concurrent connection limit reached, ",
+          "pausing accept"
+        );
+        $_[HEAP]->{listener}->pause_accept()
+      }
+    }
+    $orig_accept_callback->(@_);
+  };
+
   # Create the session, at long last.  This is done inline so that
   # closures can customize it.
 
-  $session_type->create(
+  $accept_session = $session_type->create(
     @$session_params,
     inline_states => {
       _start => sub {
@@ -328,6 +334,9 @@ sub new {
           $_[KERNEL]->alias_set( $alias );
         }
 
+        $_[HEAP]->{concurrency} = $concurrency;
+        $_[HEAP]->{connections} = 0;
+
         $_[HEAP]->{listener} = POE::Wheel::SocketFactory->new(
           ( ($domain == AF_UNIX or $domain == PF_UNIX)
             ? ()
@@ -347,6 +356,48 @@ sub new {
       # We accepted a connection.  Do something with it.
       tcp_server_got_connection => $accept_callback,
 
+      # conncurrency on close.
+      disconnected => sub {
+        $_[HEAP]->{connections}--;
+        DEBUG and warn(
+          "$$: $_[HEAP]->{alias} Connection closed ",
+          "($_[HEAP]->{connections} open)"
+        );
+        if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
+          if( $_[HEAP]->{connections} == ($_[HEAP]->{concurrency}-1) ) {
+            DEBUG and warn(
+              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
+              "reestablished, resuming accept"
+            );
+            $_[HEAP]->{listener}->resume_accept();
+          }
+        }
+      },
+
+      set_concurrency => sub {
+        $_[HEAP]->{concurrency} = $_[ARG0];
+        DEBUG and warn(
+          "$$: $_[HEAP]->{alias} Concurrent connection ",
+          "limit = $_[HEAP]->{concurrency}"
+        );
+        if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
+          if( $_[HEAP]->{connections} >= $_[HEAP]->{concurrency} ) {
+            DEBUG and warn(
+              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
+              "reached, pausing accept"
+            );
+            $_[HEAP]->{listener}->pause_accept()
+          }
+          else {
+            DEBUG and warn(
+              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
+              "reestablished, resuming accept"
+            );
+            $_[HEAP]->{listener}->resume_accept();
+          }
+        }
+      },
+
       # Shut down.
       shutdown => sub {
         delete $_[HEAP]->{listener};
@@ -355,37 +406,105 @@ sub new {
       },
 
       # Dummy states to prevent warnings.
-      _stop   => sub { return 0 },
+      _stop   => sub {
+        DEBUG and warn "$$: $_[HEAP]->{alias} _stop";
+        undef($accept_session);
+        return 0;
+      },
       _child  => sub { },
     },
 
     args => $args,
   );
 
+  $accept_session = $accept_session->ID;
+
   # Return undef so nobody can use the POE::Session reference.  This
   # isn't very friendly, but it saves grief later.
+  # Maybe we can at least return the session ID?
   undef;
 }
 
+sub _get_filters {
+    my ($client_filter, $client_infilter, $client_outfilter) = @_;
+    if (defined $client_infilter or defined $client_outfilter) {
+      return (
+        "InputFilter"  => _load_filter($client_infilter),
+        "OutputFilter" => _load_filter($client_outfilter)
+      );
+      if (defined $client_filter) {
+        carp "Using ClientFilter with ClientInputFilter or ClientOutputFilter is a noop";
+      }
+    }
+    elsif (defined $client_filter) {
+     return ( "Filter" => _load_filter($client_filter) );
+    }
+    else {
+      return ( Filter => POE::Filter::Line->new(), );
+    }
+
+}
+# Get something: either arrayref, ref, or string
+# Return filter
+sub _load_filter {
+    my $filter = shift;
+    if (ref ($filter) eq 'ARRAY') {
+        my @args = @$filter;
+        $filter = shift @args;
+        if ( _test_filter($filter) ){
+            return $filter->new(@args);
+        } else {
+            return POE::Filter::Line->new(@args);
+        }
+    }
+    elsif (ref $filter) {
+        return $filter->clone();
+    }
+    else {
+        if ( _test_filter($filter) ) {
+            return $filter->new();
+        } else {
+            return POE::Filter::Line->new();
+        }
+    }
+}
+
+# Test if a Filter can be loaded, return sucess or failure
+sub _test_filter {
+    my $filter = shift;
+    my $eval = eval {
+        (my $mod = $filter) =~ s!::!/!g;
+        require "$mod.pm";
+        1;
+    };
+    if (!$eval and $@) {
+        carp(
+          "Failed to load [$filter]\n" .
+          "Reason $@\nUsing defualt POE::Filter::Line "
+        );
+        return 0;
+    }
+    return 1;
+}
+
 # The default server error handler logs to STDERR and shuts down the
 # server.
 
 sub _default_server_error {
-  warn(
+  warn("$$: ".
     'Server ', $_[SESSION]->ID,
     " got $_[ARG0] error $_[ARG1] ($_[ARG2])\n"
   );
   delete $_[HEAP]->{listener};
 }
 
-# The default client error handler logs to STDERR and shuts down the
-# server.
+# The default client error handler logs to STDERR
 
 sub _default_client_error {
   my ($syscall, $errno, $error) = @_[ARG0..ARG2];
   unless ($syscall eq "read" and ($errno == 0 or $errno == ECONNRESET)) {
     $error = "(no error)" unless $errno;
-    warn(
+    warn("$$: ".
       'Client session ', $_[SESSION]->ID,
       " got $syscall error $errno ($error)\n"
     );
@@ -404,7 +523,7 @@ POE::Component::Server::TCP - a simplified TCP server
 
   use POE qw(Component::Server::TCP);
 
-  # First form just accepts connections.
+  ### First form just accepts connections.
 
   POE::Component::Server::TCP->new(
     Port     => $bind_port,
@@ -416,7 +535,7 @@ POE::Component::Server::TCP - a simplified TCP server
     Error    => \&error_handler,  # Optional.
   );
 
-  # Second form accepts and handles connections.
+  ### Second form also handles connections.
 
   POE::Component::Server::TCP->new(
     Port     => $bind_port,
@@ -426,6 +545,7 @@ POE::Component::Server::TCP - a simplified TCP server
     Alias    => $session_alias,     # Optional.
     Error    => \&error_handler,    # Optional.
     Args     => [ "arg0", "arg1" ], # Optional.
+    Concurrency => -1,              # Optional.
 
     SessionType   => "POE::Session::Abc",           # Optional.
     SessionParams => [ options => { debug => 1 } ], # Optional.
@@ -446,7 +566,7 @@ POE::Component::Server::TCP - a simplified TCP server
     ObjectStates  => [ ... ],
   );
 
-  # Call signatures for handlers.
+  ### Call signatures for handlers.
 
   sub accept_handler {
     my ($socket, $remote_address, $remote_port) = @_[ARG0, ARG1, ARG2];
@@ -476,7 +596,7 @@ POE::Component::Server::TCP - a simplified TCP server
     # no special parameters
   }
 
-  # Reserved HEAP variables:
+  ### Reserved HEAP variables:
 
   $heap->{listener}    = SocketFactory (only Acceptor and Error callbacks)
   $heap->{client}      = ReadWrite     (only in ClientXyz callbacks)
@@ -486,12 +606,18 @@ POE::Component::Server::TCP - a simplified TCP server
   $heap->{shutdown}    = shutdown flag (check to see if shutting down)
   $heap->{shutdown_on_error} = Automatically disconnect on error.
 
-  # Accepted public events.
+  ### Accepted public events.
 
-  $kernel->yield( "shutdown" )           # initiate shutdown in a connection
-  $kernel->post( server => "shutdown" )  # stop listening for connections
+  # Start shutting down this connection.
+  $kernel->yield( "shutdown" );
 
-  # Responding to a client.
+  # Stop listening for connections.
+  $kernel->post( server => "shutdown" );
+
+  # Set the maximum number of simultaneous connections.
+  $kernel->call( server => set_concurrency => $count );
+
+  ### Responding to a client.
 
   $heap->{client}->put(@things_to_send);
 
@@ -768,6 +894,41 @@ Its parameters are the usual for a session's _start handler.
 
 Started is optional.
 
+=item Concurrency => SCALAR
+
+Controls the number of connections that may be open at the same time.
+Defaults to -1, which means unlimited number of simutaneous connections.
+0 means no connections.  This value may be set via the
+C<set_concurrency> event, see L<EVENTS>.
+
+Note that if you define the C<Acceptor> callback, you will have to inform
+the TCP server session that a connection was closed. This is done by sending
+a C<disconnected> event to your session's parent. This is only necessary if
+you define an C<Acceptor> callback.  For C<ClientInput>, it's all handled
+for you.
+
+Example:
+
+  Acceptor => sub {
+    # ....
+    POE::Session->create(
+      # ....
+      inline_states => {
+        _start => sub {
+          # ....
+          # remember who our parent is
+          $_[HEAP]->{server_tcp} = $_[SENDER]->ID;
+          # ....
+        },
+        got_client_disconnect => sub {
+          # ....
+          $_[KERNEL]->post( $_[HEAP]->{server_tcp} => 'disconnected' );
+          # ....
+        }
+      }
+    );
+  }
+
 =back
 
 =head1 EVENTS
@@ -785,6 +946,17 @@ if one is set.
 
 Active connections are not shut down until they disconnect.
 
+=item disconnected
+
+Inform the TCP server that a connection was closed.  This is only necessary
+when using C<Concurrency> is set and you are using an L<Acceptor> callback.
+
+=item set_concurrency
+
+Set the number of simultaneous connections.  See L<Concurrency> above.
+
+  $kernel->call( "tcp_server_alias", "set_concurrency", $max_count );
+
 =back
 
 =head1 SEE ALSO
